{------------------------------------------------------------------------------}
{ TWAIN Sample Source                                                          }
{ Copyright (C) '91-'93 TWAIN Working Group:                                   }
{ Aldus, Caere, Eastman-Kodak, Logitech,                                       }
{ Hewlett-Packard Corporations.                                                }
{ All rights reserved.                                                         }
{                                                                              }
{ This source code was converted from C to Turbo Pascal for Windoes by         }
{                                                                              }
{   MCM Design                                                                 }
{   Marc Christopher Martin                                                    }
{   Australiensvej 34                                                          }
{   2100 Copenhagen OE                                                         }
{   Denmark                                                                    }
{                                                                              }
{   e-mail support@mcm-design.dk                                               }
{                                                                              }
{------------------------------------------------------------------------------}

unit twd_main;

{------------------------------------------------------------------------------}
{$IFDEF WIN32}
  {$A-} { Align data Byte = $A-, Word = $A+.                                   }
{$ELSE}
  {$A+} { Align data Byte = $A-, Word = $A+.                                   }
{$ENDIF}
{$F+} { Force FAR call.                                                        }
{$N-} { CoProcessor code generation.                                           }
{$R-} { Range check.                                                           }
{$W-} { Protected mode only $W-, Real mode stack frame support $W+.            }
{$X+} { Extended syntax (When on, function can be used as procedures).         }
{------------------------------------------------------------------------------}

{$C PRELOAD MOVEABLE DISCARDABLE}

interface

uses WinTypes, WinProcs, Dialogs,
     twain, twd_res, twd_type;

{------------------------------------------------------------------------------}
{ Extern Function Prototypes                                                   }
{------------------------------------------------------------------------------}

{$IFDEF WIN32}

function LibMain(hLibModule     : HModule;
	         ulReasonCalled : DWORD;
		 lpReserved     : pointer) : BOOL;

function DS_Entry(pSrc  : pTW_IDENTITY;
                  DG    : TW_UINT32;
		  DAT   : TW_UINT16;
                  MSG   : TW_UINT16;
		  pData : TW_MEMREF) : TW_UINT16; stdcall; export;

{$ELSE}

function LibMain(hLibInstance : THandle;
                 wDataSeg     : word;
                 wHeapSize    : word;
                 lpCmdLine    : PChar) : bool; far;

function DS_Entry(pSrc  : pTW_IDENTITY;
                  DG    : TW_UINT32;
		  DAT   : TW_UINT16;
                  MSG   : TW_UINT16;
		  pData : TW_MEMREF) : TW_UINT16; export;

{$ENDIF}


const NUM_PIXTYPES = 4; { Size of number of pixel types array.                 }

      { Support for pixel types TW_ARRAY container.                            }
      PixelTypesList    : array[0..NUM_PIXTYPES-1] of TW_UINT16
                        = (TWPT_BW, TWPT_GRAY, TWPT_RGB, TWPT_PALETTE);
      PixelTypesTWArray : TW_ARRAY
                        = (ItemType : TWTY_UINT16;
                           NumItems : NUM_PIXTYPES;
                           ItemList : (57));

{ Global variables                                                             }
const TimerOut      = 999;      { timeout value for message boxes.             }

var   SourceControl : array[0..MAX_TWPATH] of char; {= {'\0';                  }

const twStatus      : TW_STATUS
                    = (ConditionCode : TWCC_SUCCESS;
                       Reserved      : (0));

implementation

uses twd_prot;

{$IFDEF WIN32}

{------------------------------------------------------------------------------}
{ FUNCTION: LibMain                                                            }
{                                                                              }
{                                                                              }
{ ARGS:     hLibModule      handle to current instance                         }
{           ulReasonCalled                                                     }
{           lpReserved                                                         }
{                                                                              }
{ RETURNS:  1                                                                  }
{                                                                              }
{ NOTES:    This function intializes the Source DLL                            }
{------------------------------------------------------------------------------}

function LibMain(hLibModule     : HModule;
	         ulReasonCalled : DWORD;
		 lpReserved     : pointer) : bool;
var iError : integer;
begin
  if (hLibModule <> 0)
  then begin
       case ulReasonCalled of
       DLL_PROCESS_ATTACH : begin
			      { Initialize any necessary globals.              }
	   		      hDSInst := hLibModule;

   			      { state 3                                        }

   			      { Clear out calling app's identity.              }
   			      appIdentity.Id := 0;

   			      { Setup Source's identity and group supported.   }
                              with dsIdentity
                              do begin
   			         Id               := 0;
			         ProtocolMajor    := TWON_PROTOCOLMAJOR;
			         ProtocolMinor    := TWON_PROTOCOLMINOR;
   			         Version.MajorNum := 1;
  			         Version.MinorNum := 0;
			         Version.Language := TWLG_ENG;
			         Version.Country  := TWCY_UNITEDKINGDOM;
   			         SupportedGroups  := DG_CONTROL or DG_IMAGE;
                              end;
                              iError := LoadString(hDSInst, VERSION_INFO, dsIdentity.Version.Info,  sizeof(TW_STR32));
			      iError := LoadString(hDSInst, MANUFACTURER, dsIdentity.Manufacturer,  sizeof(TW_STR32));
			      iError := LoadString(hDSInst, PRODFAMILY,   dsIdentity.ProductFamily, sizeof(TW_STR32));
			      iError := LoadString(hDSInst, PRODNAME,     dsIdentity.ProductName,   sizeof(TW_STR32));
                            end;
       DLL_THREAD_ATTACH  : begin
                            end;
       DLL_THREAD_DETACH  : begin
                            end;
       DLL_PROCESS_DETACH : begin
                            end;
       end;
       LibMain := True;
  end
  else LibMain := False;
end;  { End LibMain.                                                           }


{$ELSE} { GR: 16 bit version.                                                  }


{------------------------------------------------------------------------------}
{ FUNCTION: LibMain                                                            }
{                                                                              }
{ ARGS:    hInstance   handle to current instance                              }
{          wDataSeg                                                            }
{          wHeapSize                                                           }
{          lpCmdLine                                                           }
{                                                                              }
{ RETURNS: 1                                                                   }
{                                                                              }
{ NOTES:   This function intializes the Source DLL                             }
{------------------------------------------------------------------------------}

function LibMain(hLibInstance : THandle;
                 wDataSeg     : word;
                 wHeapSize    : word;
                 lpCmdLine    : PChar) : bool;
begin
  if (hLibInstance <> 0)
  then begin
{------------------------------------------------------------------------------}
{ LocalInit() adds one lock to the Data Segment.                               }
{ We call UnlockSegment() here to remove this lock and                         }
{ allow the Source to move freely.                                             }
{                                                                              }
{ WARNING 1:                                                                   }
{   Any function call outside the current code segment may move the Source.    }
{ WARNING 2:                                                                   }
{   LocalAlloc() and LocalRealloc() will unlock and move the Source, if        }
{   need be.                                                                   }
{------------------------------------------------------------------------------}

       if (wHeapSize > 0)
       then UnlockSegment(wDataSeg);

       { Initialize any necessary globals.                                     }
       hDSInst := hInstance;

       { state 3                                                               }

       { Clear out calling app's identity                                      }
       appIdentity.Id := longint(0);

       { Setup Source's identity and group supported.                          }
       with dsIdentity
       do begin
          Id               := 0;
          ProtocolMajor    := TWON_PROTOCOLMAJOR;
          ProtocolMinor    := TWON_PROTOCOLMINOR;
          Version.MajorNum := 1;
          Version.MinorNum := 0;
          Version.Language := TWLG_ENG;
          Version.Country  := TWCY_UNITEDKINGDOM;
          SupportedGroups  := DG_CONTROL or DG_IMAGE;
       end;
       LoadString(hDSInst, VERSION_INFO, dsIdentity.Version.Info,  sizeof(TW_STR32));
       LoadString(hDSInst, MANUFACTURER, dsIdentity.Manufacturer,  sizeof(TW_STR32));
       LoadString(hDSInst, PRODFAMILY,   dsIdentity.ProductFamily, sizeof(TW_STR32));
       LoadString(hDSInst, PRODNAME,     dsIdentity.ProductName,   sizeof(TW_STR32));

       LibMain := True;
  end
  else LibMain := False;
end; { End LibMain.                                                            }

{$ENDIF} { WIN32.                                                              }


{------------------------------------------------------------------------------}
{ FUNCTION: DS_Entry                                                           }
{                                                                              }
{ ARGS:    pSrc    ptr to caller, source of message                            }
{          DG      data group ID, DG_IMAGE, DG_xxxx                            }
{          DAT     data argument type, DAT_xxxx                                }
{          MSG     message ID, MSG_xxxx                                        }
{          pData   pointer to the data                                         }
{                                                                              }
{ RETURNS: twRC    status code                                                 }
{                                                                              }
{ NOTES:   This function is declared as the single entry point to the Source.  }
{          It should be exported as Ordinal function 1 (@1).                   }
{------------------------------------------------------------------------------}

{$IFDEF WIN32}
  function DS_Entry(pSrc  : pTW_IDENTITY;
                    DG    : TW_UINT32;
                    DAT   : TW_UINT16;
                    MSG   : TW_UINT16;
                    pData : TW_MEMREF) : TW_UINT16; stdcall;
{$ELSE}
  function DS_Entry(pSrc  : pTW_IDENTITY;
                    DG    : TW_UINT32;
                    DAT   : TW_UINT16;
                    MSG   : TW_UINT16;
                    pData : TW_MEMREF) : TW_UINT16;
{$ENDIF}
var twMsg : TTWMessage;
    twRc  : TW_UINT16;
begin
{------------------------------------------------------------------------------}
{ TW apps are sending down all their messages prior to process them            }
{ to give us a chance to extract messages such as dialog keyboard              }
{ handling, accelerator translation or MDI processing in a Source              }
{ while the Source is enabled. We are doing this filtering prior to process    }
{ real Source messages to minimize overhead.                                   }
{ Relay windows messages down to the Source via the SM.  The SM will send      }
{ a MSG_PROCESSEVENT to the Source to flag the arrival of a message relayed    }
{ from the APP that was originally a windows message.                          }
{                                                                              }
{ Valid states: 5 -- 7.  As soon as the application has enabled the Source it  }
{ must being sending the Source events.  This allows the Source to recieve     }
{ events to update it's user interface and to return messages to the           }
{ application.                                                                 }
{ The app sends down ALL message, the Source decides which ones apply to it.   }
{ added to keep stop Apps from talking to us until properly connected          }
{ NOTE: Since SM peeks into Sources pSrc may be invalid (NULL) on some calls   }
{ so the order of evaluation is key.                                           }
{------------------------------------------------------------------------------}
  if ((appIdentity.Id = 0) or (appIdentity.Id = pSrc^.Id))
  then begin
       if (MSG = MSG_PROCESSEVENT)
       then begin
            LogMessage('Process event message at source');

            if (hImageDlg <> 0) and
               IsDialogMessage(hImageDlg, TMsg(pTW_EVENT(pData)^.pEvent^))
            then begin
	         LogMessage('Process event Message directed to data source dialog');
	         twRc := TWRC_DSEVENT;
                 { you should, for proper form, return                         }
                 { a MSG_NULL for all windows messages processed by the        }
                 { Data Source.                                                }
                 { NOTE: could slip by with a default case in the App and      }
                 {       no MSG_NULL but that would be cheating!!              }
	         pTW_EVENT(pData)^.TWMessage := MSG_NULL;
            end
            else begin
	         LogMessage('Process event not ds event');
	         twRc := TWRC_NOTDSEVENT;
            end;
       end
       else begin
            { Reset the condition code.                                        }
            if (DG  <> DG_CONTROL) and
               (DAT <> DAT_STATUS)
            then twStatus.ConditionCode := TWCC_SUCCESS;

            { Build the message for the dispatch routine.                      }
  	    twMsg.pSrc  := pSrc;
	    twMsg.DG    := DG;
    	    twMsg.DAT   := DAT;
	    twMsg.MSG   := MSG;
	    twMsg.pData := pData;

            { Route the Source message.                                        }
            if (DG = DG_CONTROL)
            then twRc := twd_prot.Dispatch(@twMsg, PMsgMap(@ControlMap), DAT)
            else begin
                 if (DG = DG_IMAGE)
                 then twRc := twd_prot.Dispatch(@twMsg, PMsgMap(@ImageMap), DAT)
                 else begin
	              twStatus.ConditionCode := TWCC_BADPROTOCOL;
		      twRc := TWRC_FAILURE;
                 end;
            end;
       end;
  end;
  DS_Entry := twRc;
end; { End DS_Entry.                                                           }

end. { End Unit twd_main.                                                      }

